% Run this file for complete evaluation of all the models.
% Some of the models take long time to run, so adjust.
clear all;
close all;
%%% Set the main folder path %%%
folder_path = 'ScratchStimuliTest'
addpath(genpath(folder_path))
folder_path = 'MAP'
addpath(genpath(folder_path))
folder_path = 'IR'
addpath(genpath(folder_path))
folder_path = 'fce_wavelet'
addpath(genpath(folder_path))
folder_path = 'fce_matched'
addpath(genpath(folder_path))
folder_path = 'fce_audmodel'
addpath(genpath(folder_path))
folder_path = 'fce_AR'
addpath(genpath(folder_path))
folder_path = 'amtoolbox-full-1.2.0'
addpath(genpath(folder_path))

% SETUP
% Experiment Selection 
exp = 1  % 2, 3;

% linear threshold division no
% 50 for Vencovsky model and 300 for other models 
% The change has done manually. Depending on the PC specifications
% Can be made equal. However, vencovsky model takes long time to process.
%%%%% Uncomment the below codes accordingly to run Vencovsky model or
%%%%% others, and arrange n accordingly.
n = 1 ;


%%%%%%%%%%%%%
if exp == 1
    % prepare train and test lists
    sig_num = 90; % train list size
    number_trials = 1; % number of runs
else 
    % prepare train and test lists
    sig_num = 60; % train list size
    number_trials = 10; % number of randomization because Exp 2 and 3 take random sets
end

% [75,25] = [H, F] listening test thresholds bigger than H value is taken as
% H, and smaller then F value is taken as F.
if exp == 2
    listening_test_th = [50,50]
else
    listening_test_th = [75,25]
end



tic;
% INITIALIZATIONS
%new models
th_drnl = linspace(10, 32, n);
th_erb = linspace(3, 10, n);
th_lyon = linspace(8, 22, n);
th_senef = linspace(2, 20, n);

%old models
th_ar = linspace(4, 32, n);
th_matched = linspace(5, 50, n);
th_wave = linspace(5, 50, n);
th_vencovs = linspace(25, 65, n);

% train set results
Ap_train = zeros(8,n); % Aprime results matrix)
dprime_train = zeros(8,n); % dprime results matrix)
cd_train = zeros(8,number_trials,n); % Correct detection ratio matrix)
fd_train = zeros(8,number_trials,n); % False detection results matrix)

% test set results
% third dimension ==> 1 = Ap results,,,, 2 = dp results
Ap_test = zeros(8,number_trials,2); % Aprime results matrix)
dprime_test = zeros(8,number_trials,2); % dprime results matrix)
cd_test = zeros(8,number_trials,2); % Correct detection ratio matrix)
fd_test = zeros(8,number_trials,2); % False detection results matrix)

% time measurements (NOTE: STRUCTURE OF THE TIME MEASUREMENTS IN EXPERIMENT
% 1 ARE DIFFERENT THEN THE ONE INCLUDED HERE BECAUSE THOSE TESTS WERE DONE
% EARLIER. THIS CODE HERE IS REFINED FOR FUTURE USE)
elapsed_time = zeros(8, number_trials,n+2);

% max_vals and indexes for train
max_train = zeros(8,number_trials,2);
maxidx_train = zeros(8,number_trials,2);

thresholds = struct('drnl',th_drnl,...
                    'erb',th_erb,...
                    'lyon',th_lyon,...
                    'senef',th_senef,...
                    'ar',th_ar,...
                    'matched',th_matched,...
                    'wave',th_wave,...
                    'vencovs',th_vencovs);

train_lists_all = zeros(number_trials, sig_num);                

% run for randomizations (exp 2 3)
for i= 1:number_trials;
if exp == 1
    train_list = [1:90];
    train_lists_all(i,:) = train_list;
    test_list = train_list;
else
    rng('shuffle');  % set the random number generator to use the current time as the seedt
    train_list = randperm(90, sig_num); % random test list
    train_lists_all(i,:) = train_list;
    test_list = setdiff([1:90], train_list);
end
    % run for thresholds
    for t = 1:n;
        [Ap_train(1,t),dprime_train(1,t),cd_train(1,i,t),fd_train(1,i,t),elapsed_time(1,i,t)] = run_detect_DRNL(th_drnl(t),train_list,listening_test_th);
        [Ap_train(2,t),dprime_train(2,t),cd_train(2,i,t),fd_train(2,i,t),elapsed_time(1,i,t)] = run_detect_erblet(th_erb(t),train_list,listening_test_th);
        [Ap_train(3,t),dprime_train(3,t),cd_train(3,i,t),fd_train(3,i,t),elapsed_time(1,i,t)] = run_detect_Lyon(th_lyon(t),train_list,listening_test_th);
        [Ap_train(4,t),dprime_train(4,t),cd_train(4,i,t),fd_train(4,i,t),elapsed_time(1,i,t)] = run_detect_Seneff(th_senef(t),train_list,listening_test_th);
        [Ap_train(5,t),dprime_train(5,t),cd_train(5,i,t),fd_train(5,i,t),elapsed_time(1,i,t)] = runAnalysisAR_JAES_f(th_ar(t),train_list,listening_test_th);
        [Ap_train(6,t),dprime_train(6,t),cd_train(6,i,t),fd_train(6,i,t),elapsed_time(1,i,t)] = runAnalysismatch_JAES_f(th_matched(t),train_list,listening_test_th);
        [Ap_train(7,t),dprime_train(7,t),cd_train(7,i,t),fd_train(7,i,t),elapsed_time(1,i,t)] = runAnalysisWavelet_JAES_f(th_wave(t),train_list,listening_test_th);
         [Ap_train(8,t),dprime_train(8,t),cd_train(8,i,t),fd_train(8,i,t),elapsed_time(1,i,t)] = runBMmodelTEST_JAES_2(th_vencovs(t),train_list,listening_test_th);
    end
    
    % find the best Ap and dp thresholds
    % max_...(:,1) = Ap
    [max_train(1,i,1), maxidx_train(1,i,1)] = max(Ap_train(1,:));
    [max_train(2,i,1), maxidx_train(2,i,1)] = max(Ap_train(2,:));
    [max_train(3,i,1), maxidx_train(3,i,1)] = max(Ap_train(3,:));
    [max_train(4,i,1), maxidx_train(4,i,1)] = max(Ap_train(4,:));
    [max_train(5,i,1), maxidx_train(5,i,1)] = max(Ap_train(5,:));
    [max_train(6,i,1), maxidx_train(6,i,1)] = max(Ap_train(6,:));
    [max_train(7,i,1), maxidx_train(7,i,1)] = max(Ap_train(7,:));
    [max_train(8,i,1), maxidx_train(8,i,1)] = max(Ap_train(8,:));
    
    % max_...(:,2) = dp %just as a try 
    [max_train(1,i,2), maxidx_train(1,i,2)] = max(dprime_train(1,:));
    [max_train(2,i,2), maxidx_train(2,i,2)] = max(dprime_train(2,:));
    [max_train(3,i,2), maxidx_train(3,i,2)] = max(dprime_train(3,:));
    [max_train(4,i,2), maxidx_train(4,i,2)] = max(dprime_train(4,:));
    [max_train(5,i,2), maxidx_train(5,i,2)] = max(dprime_train(5,:));
    [max_train(6,i,2), maxidx_train(6,i,2)] = max(dprime_train(6,:));
    [max_train(7,i,2), maxidx_train(7,i,2)] = max(dprime_train(7,:));
    [max_train(8,i,2), maxidx_train(8,i,2)] = max(dprime_train(8,:));
    
    
    % Ap best threshold test set results
    [Ap_test(1,i,1),dprime_test(1,i,1),cd_test(1,i,1),fd_test(1,i,1),elapsed_time(1,i,n+1)] = run_detect_DRNL(th_drnl(maxidx_train(1,i,1)),test_list,listening_test_th);
    [Ap_test(2,i,1),dprime_test(2,i,1),cd_test(2,i,1),fd_test(2,i,1),elapsed_time(1,i,n+1)] = run_detect_erblet(th_erb(maxidx_train(2,i,1)),test_list,listening_test_th);
    [Ap_test(3,i,1),dprime_test(3,i,1),cd_test(3,i,1),fd_test(3,i,1),elapsed_time(1,i,n+1)] = run_detect_Lyon(th_lyon(maxidx_train(3,i,1)),test_list,listening_test_th);
    [Ap_test(4,i,1),dprime_test(4,i,1),cd_test(4,i,1),fd_test(4,i,1),elapsed_time(1,i,n+1)] = run_detect_Seneff(th_senef(maxidx_train(4,i,1)),test_list,listening_test_th);
    [Ap_test(5,i,1),dprime_test(5,i,1),cd_test(5,i,1),fd_test(5,i,1),elapsed_time(1,i,n+1)] = runAnalysisAR_JAES_f(th_ar(maxidx_train(5,i,1)),test_list,listening_test_th);
    [Ap_test(6,i,1),dprime_test(6,i,1),cd_test(6,i,1),fd_test(6,i,1),elapsed_time(1,i,n+1)] = runAnalysismatch_JAES_f(th_matched(maxidx_train(6,i,1)),test_list,listening_test_th);
    [Ap_test(7,i,1),dprime_test(7,i,1),cd_test(7,i,1),fd_test(7,i,1),elapsed_time(1,i,n+1)] = runAnalysisWavelet_JAES_f(th_wave(maxidx_train(7,i,1)),test_list,listening_test_th);
    % [Ap_test(8,i,1),dprime_test(8,i,1),cd_test(8,i,1),fd_test(8,i,1),elapsed_time(1,i,n+1)] = runBMmodelTEST_JAES_2(th_vencovs(maxidx_train(8,i,1)),test_list,listening_test_th);
    
    % dp best threshold test set results
    [Ap_test(1,i,2),dprime_test(1,i,2),cd_test(1,i,2),fd_test(1,i,2),elapsed_time(1,i,n+2)] = run_detect_DRNL(th_drnl(maxidx_train(1,i,2)),test_list,listening_test_th);
    [Ap_test(2,i,2),dprime_test(2,i,2),cd_test(2,i,2),fd_test(2,i,2),elapsed_time(1,i,n+2)] = run_detect_erblet(th_erb(maxidx_train(2,i,2)),test_list,listening_test_th);
    [Ap_test(3,i,2),dprime_test(3,i,2),cd_test(3,i,2),fd_test(3,i,2),elapsed_time(1,i,n+2)] = run_detect_Lyon(th_lyon(maxidx_train(3,i,2)),test_list,listening_test_th);
    [Ap_test(4,i,2),dprime_test(4,i,2),cd_test(4,i,2),fd_test(4,i,2),elapsed_time(1,i,n+2)] = run_detect_Seneff(th_senef(maxidx_train(4,i,2)),test_list,listening_test_th);
    [Ap_test(5,i,2),dprime_test(5,i,2),cd_test(5,i,2),fd_test(5,i,2),elapsed_time(1,i,n+2)] = runAnalysisAR_JAES_f(th_ar(maxidx_train(5,i,2)),test_list,listening_test_th);
    [Ap_test(6,i,2),dprime_test(6,i,2),cd_test(6,i,2),fd_test(6,i,2),elapsed_time(1,i,n+2)] = runAnalysismatch_JAES_f(th_matched(maxidx_train(6,i,2)),test_list,listening_test_th);
    [Ap_test(7,i,2),dprime_test(7,i,2),cd_test(7,i,2),fd_test(7,i,2),elapsed_time(1,i,n+2)] = runAnalysisWavelet_JAES_f(th_wave(maxidx_train(7,i,2)),test_list,listening_test_th);
    % [Ap_test(8,i,2),dprime_test(8,i,2),cd_test(8,i,2),fd_test(8,i,2),elapsed_time(1,i,n+2)] = runBMmodelTEST_JAES_2(th_vencovs(maxidx_train(8,i,2)),test_list,listening_test_th);
    
end
ALLTIME = toc;
disp(ALLTIME)

save('REV1_results.mat','n'...
                       ,'sig_num'...
                       ,'number_trials'...
                       ,'train_lists_all'...
                       ,'cd_train'...
                       ,'fd_train'...
                       ,'max_train'...
                       ,'maxidx_train'...
                       ,'dprime_test'...
                       ,'Ap_test'...
                       ,'cd_test'...
                       ,'fd_test'...
                       ,'thresholds'...
                       ,'elapsed_time'...
                       );

