% matched filter based click-detection test
% Authors: Vclav Vencovsk, Marek Semansk, Frantisek Rund
% FEE CTU 2021
% MATLAB Version: 9.4.0.813654 (R2018a)

%clear, close all, %clc
function [Ap, dprime, CD, FA, elapsed_time] = runAnalysismatch_JAES_f(threshold_gain,sig_list,listening_test_th)

% addpath('fce_matched');  % add path to folder with matched filter click detection functions

%sig_list = list of signals to be used
% listening_test_th = [H F] listening test thresholds 



%subjective test results loading
load('subj_results.mat');
subjRes = vysledky(sig_list,2);


idxHIT = find(subjRes>=listening_test_th(1)); %marked as containing clicks 
idxNO = find(subjRes<=listening_test_th(2));  

%Folder with the samples
FolderName = 'ScratchStimuliTest/';
listWav = dir(FolderName);

%detection parameters
LengthSeg = 6144; %Frame length N !
Threshold = threshold_gain % 35; %Detection threshold gain K !
Order = 30; %Prediction order P

idxLup = 17640; % Starting position for the detection - half of the signal (sample) length
idxTol = idxLup-1; % Range for click detection - half of the signal (sample) length - a click is searched in full length of the sample

%kstim = sort([idxHIT;idxNO]); %"Unsure" samples removed (25 % - 75 %)
kstim=sig_list; %all samples
disp(['Frame length N: ' num2str(LengthSeg) ', Detection threshold parameter K: ' num2str(Threshold) ', Prediction order p: ' num2str(Order)]) %zobrazeni parametru detekce

clickAR = zeros(1,length(listWav)-2); %preallocation of detection vector


tic
%-------------------------------------------------------------------------------------------------
for k=1:length(kstim) %%detection for cycle
    %-------------------------------------------------------------------------------------------------
    disp(['stim: ' num2str(kstim(k))])
    [y, fs] = audioread([FolderName listWav(kstim(k)+2).name]); %sample loading

    y = y(:,1); %only left channel used for the subjective test 
    
    if min(abs(idxHIT-kstim(k)))==0 %matched filter
          
          clickAR(kstim(k)) = match_filter_mm(y, Order, Threshold, LengthSeg,idxLup,idxTol,1);%click perceived
    else
          clickAR(kstim(k)) = match_filter_mm(y, Order, Threshold, LengthSeg,idxLup,idxTol,0);%click not perceived
    end    
end
elapsed_time = toc; %time elapsed
%evaluation
ErrorRate = 100-(sum(clickAR(idxHIT))/length(idxHIT))*100; 
CD=100-ErrorRate; %Hit
FA = sum(abs(clickAR(idxNO)))/length(idxNO)*100; % False Alarm
total = ErrorRate + FA;
dprime=(norminv(CD/100)-norminv(FA/100));
% Ap
cd = CD/100;
fd = FA/100;
Ap = 0.5 + sign(cd-fd)*((cd-fd)^2 + abs(cd-fd))/(4*max(cd,fd)-4*cd*fd);

disp(['Not detected: ' num2str(ErrorRate) '%'])
disp(['Correcty detected: ' num2str(CD) '%'])
disp(['False detected: ' num2str(FA) '%'])
disp(['d prime: ' num2str(dprime) ])
disp(['A prime: ' num2str(Ap) ])
disp(['Time: ' num2str(elapsed_time) ' sec'])
disp(['----------------------------'])


% figure
% bar(idxHIT,clickAR(idxHIT))
% hold on
% bar(idxNO,abs(clickAR(idxNO)))
% plot(idxHIT,subjRes(idxHIT),'-*')
% plot(idxNO,subjRes(idxNO),'-*')
% title('matched')

% rmpath('fce_matched');