% Impulse noise detection using ear model
% Lyon model based detection algorithm - evaluation script
% Authors: Marek Semansky, Frantisek Rund
% FEE CTU in Prague
% https://mmtg.fel.cvut.cz/click-detection/

% R. F. Lyon, A Computational Model of Filtering, Detection and
% Compression in the Cochlea, v Proceedings of IEEE International
% Conference on Acoustics, Speech and Signal Processing, Paris, 1982

%Lyon needs M. Slaney Auditory Toolbox
%https://engineering.purdue.edu/~malcolm/interval/1998-010/ (tested at ver 2)

function [Ap, dprime, results_perc, false_perc, elapsed_time] = run_detect_Lyon(threshold_gain,sig_list,listening_test_th)
%clear all%, close all, clc
clear det dprime elapsed_time false_hit false_perc hit num_sig results_perc sada start_sig threshold vysledky%, close all, clc

%% parameters 
start_sig = 1; %first sample (sound file) for analysis (1 - 90)
% sig_list = list of signals to be used
num_sig = length(sig_list);% number of samples (sound files) for analysis (1 - 90)
% listening_test_th = [H F] listening test thresholds 


hit = zeros(1, num_sig);
false_hit = zeros(1, num_sig);  

%Input stimuli files, loading of subjective test results
% details in JAES 2021 article

sada = 'ScratchStimuliTest/'; %path to sound samples
load('subj_results.mat'); %load subjective test results

%evaluation threshold 
        hit(vysledky(sig_list, 2)>=listening_test_th(1)) = 1;    %hit
        false_hit(vysledky(sig_list, 2)<=listening_test_th(2)) = 1;  %false hit

% %subjective test results processing
% hit = hit(sig_list); %vector of signals marked by majority of subject as containing clicks
% false_hit = false_hit(sig_list); %vector of singals marked by majority of subjects as not containing clicks

%% calling the detection function
[det, elapsed_time] = JAES_lyon(sada, start_sig, num_sig, threshold_gain,sig_list);
%[det, elapsed_time] = JAES_lyonp(sada, start_sig, num_sig);

%% evaluation 
%clc


disp('Lyon')
[results_perc, false_perc, dprime, Ap] = evaluation(det, hit, false_hit, vysledky);
disp(['Correctly detected: ' num2str(results_perc) '%'])
disp(['False detected: ' num2str(false_perc) '%'])
disp(['d prime: ' num2str(dprime) ])
disp(['A prime: ' num2str(Ap) ])
disp(['Time: ' num2str(elapsed_time) ' sec'])

disp('----------------------------')

%% plot
% idxHIT=find(hit);
% idxNO=find(false_hit);
% figure
% bar(idxHIT,det(idxHIT))
% hold on
% bar(idxNO,abs(det(idxNO)))
% plot(idxHIT,vysledky(idxHIT,2)/100,'-*')
% plot(idxNO,vysledky(idxNO,2)/100,'-*')
% title('Lyon')

